/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2003
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQSeries Workflow API Examples - Handling process templates        */
/*                                                                    */
/* This sample program shows how to start process instances from an   */
/* existing process template.                                         */
/**********************************************************************/

#include <fmctjinc.hxx>                 // compiler-dependent #includes
#include <fmcjprun.hxx>                 // MQWF C++ API interface
#include <ctype.h>                      // character classification


int main (int argc, char ** argv)
{

  FmcjExecutionService                  service;

  vector<FmcjProcessTemplate>           tList;

  unsigned long                         i               = 0;
  unsigned long                         j               = 0;
  unsigned long                         threshold       = 10;

  FmcjProcessInstance                   processInstance;

  string                                choice          ="";
  string                                PTListSort      ="NAME ASC";
  int                                   usrchoice       = 0;

  FmcjReadWriteContainer                iictnr;


/*--------------------------------------------------------------------*/
/* Check command line parameters, logon to system                     */
/*--------------------------------------------------------------------*/

  if (argc!=3)
  {
    cout << "\nUsage: " << argv[0] << " <UserID> <Password>" << endl;
    return -1;
  }
  cout << endl;
  APIRET rc = FmcjGlobal::Connect();
  FmcjResult * result = FmcjResult::ObjectOfCurrentThread();
  rc = service.Logon( argv[1], argv[2], FmcjService::Default );
  if (rc != FMC_OK)
  {
    cout << "Logon failed with RC " << rc << " - " << result->MessageText()
         << endl;
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }
  cout << "Logged on with UserID '" << service.UserID()
       << "' on system '" << service.SystemName()
       << "' at system group '" << service.SystemGroupName()
       << "'." << endl;


/*--------------------------------------------------------------------*/
/* Display current template list, query user for action               */
/*--------------------------------------------------------------------*/

  do                                    // Beginning of main tool loop
  {
    // Clear template vector

    tList.erase(tList.begin(),tList.end());

    // Display as many process templates as specified with threshold

    cout << "\nAvailable process templates:\n" << endl;

    rc = service.QueryProcessTemplates( FmcjNoFilter,
                                        &PTListSort,
                                        &threshold,
                                        tList );

    if (rc != FMC_OK)
    {
      cout << "Querying process templates failed with RC " << rc << " - "
           << result->MessageText() << endl;
      service.Logoff();
      // Deinit API environment
      FmcjGlobal::Disconnect();
      delete(result);
      return -1;
    }

    cout << "    Input container\n"
         << "#   needed?           Template name\n"
         << "-----------------------------------" << endl;

    if (tList.size() > 0 )
    {
      for( i=0; i< tList.size(); i++ )
      {
        cout << i;
        if (i<10) cout << " ";
        if (tList[i].InContainerNeeded()) cout << "  yes               ";
        else cout << "  no                ";
        cout << tList[i].Name() << endl;
      }
    }
    else cout << "  <no templates available>" << endl;

    cout << endl
         << "Commands: [v] View template info                    [c] Create an instance\n"
         << "          [s] Create and start an instance          [q] Quit program\n"
         << "[v/s/c/q]? ";
    cout.flush();

    cin >> choice; cout << endl;

    usrchoice=0;

    if (strncmp(choice.c_str(),"v",1)==0) usrchoice=1;
    if (strncmp(choice.c_str(),"s",1)==0) usrchoice=2;
    if (strncmp(choice.c_str(),"c",1)==0) usrchoice=3;
    if (strncmp(choice.c_str(),"q",1)==0) usrchoice=255;

    if (tList.size()>0)

    switch (usrchoice)
    {

      case 1:
/*-------------------------------------------------------------------*/
/* Shows some attributes of the selected process template.           */
/* If secondary attributes are accessed the API refreshes the        */
/* template object automatically from the server.                    */
/*-------------------------------------------------------------------*/

        do
        {
          cout << "Please enter template number: ";
          cout.flush();
          cin >> choice;
          j=0;
          for (i=0; i<strlen(choice.c_str()); i++)
          { if (isdigit(choice.c_str()[i])) j++; }
        } while (i!=j);

        i = atoi(choice.c_str());
        cout << endl;

        if (i>(tList.size()-1) && tList.size()>0)
        {
          cout << "Please use range 0-" << tList.size()-1 << "." << endl;
        }
        else
        {
          cout << "Template properties:" << endl;
          cout << "--------------------" << endl;
          cout << "   (P) Name     : " << tList[i].Name() << endl;
          cout << "   (P) Category : " << tList[i].Category() << endl;
          cout << "   (P) Object id: " << tList[i].PersistentOid() << endl << endl;
          cout << "   (S) Process administrator: " << tList[i].ProcessAdmin() << endl;
          cout << "   (S) Input container name : " << tList[i].InContainerName() << endl;
          cout << "   (S) Output container name: " << tList[i].OutContainerName() << endl << endl;
        } /* end else */

        break;


      case 2:
/*-------------------------------------------------------------------*/
/* Demonstrates how to create and start a process instance.          */
/*-------------------------------------------------------------------*/

        do
        {
          cout << "Please enter template number: ";
          cout.flush();
          cin >> choice;
          j=0;
          for (i=0; i<strlen(choice.c_str()); i++)
          { if (isdigit(choice.c_str()[i])) j++; }
        } while (i!=j);

        i = atoi(choice.c_str());
        cout << endl;

        if (i>(tList.size()-1) && tList.size()>0)
        {
          cout << "Please use range 0-" << tList.size()-1 << "." << endl;
        }
        else                            // user's choice is in range
        {
          if (tList[i].InContainerNeeded())
          {
            // Query for input container values only for "CreditRequest"

            if (tList[i].Name()=="CreditRequest")
            {
              tList[i].InitialInContainer(iictnr);

              cout << "FirstName: ";
              cout.flush();

              cin >> choice;

              rc= iictnr.SetValue("FirstName",choice);

              cout << "LastName: ";
              cout.flush();

              cin >> choice;

              rc= iictnr.SetValue("LastName",choice);

              // End of query. Now creating and starting instance with the
              // filled initial input container.

              rc = tList[i].CreateAndStartInstance( 0, 0, 0,
                                                    iictnr,
                                                    processInstance,
                                                    false );

            }
            else
            {
              // Create and start instance without passing an input container

              rc = tList[i].CreateAndStartInstance( 0, 0, 0,
                                                    processInstance,
                                                    false );
            }

          }
          else
          {
            // Create and start instance without passing an input container

            rc = tList[i].CreateAndStartInstance( 0, 0, 0,
                                                  processInstance,
                                                  false );
          }

          // Check if instance was created and started successfully

          if (rc != FMC_OK)
          {
            cout << "Creating new instance failed with RC " << rc << " - "
                 << result->MessageText() << endl;
            service.Logoff();
            // Deinit API environment
            FmcjGlobal::Disconnect();
            delete(result);
            return -1;
          }

        } // end else: user's choice is in range

        break;


      case 3:
/*-------------------------------------------------------------------*/
/* Demonstrates how to create a process instance                     */
/*-------------------------------------------------------------------*/

        do
        {
          cout << "Please enter template number: ";
          cout.flush();
          cin >> choice;
          j=0;
          for (i=0; i<strlen(choice.c_str()); i++)
          { if (isdigit(choice.c_str()[i])) j++; }
        } while (i!=j);

        i = atoi(choice.c_str());
        cout << endl;

        if (i>(tList.size()-1) && tList.size()>0)
        {
          cout << "Please use range 0-" << tList.size()-1 << "." << endl;
        }
        else
        {
          // Create an instance without starting it

          rc = tList[i].CreateInstance( 0, 0, 0,
                                        processInstance,
                                        false );
        }

        break;

      }                                 // end of switch
      else cout << endl << "No templates available!" << endl;

  } while ( (usrchoice!=255) );         // end of main tool loop


/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/

  cout << "\nLogging off." << endl;
  rc = service.Logoff();
  cout << "Deinitializing API environment." << endl;
  rc = FmcjGlobal::Disconnect();
  delete(result);
  return 0;
}
