/*BEGIN CMVC */
/**************************************************************************
*
*   Workfile: fmctjcim.c, v3-test, fmv3
*   Last update: 00/04/19 18:44:26
*   SCCS path, id: /home/flowmark/vc/0/1/6/1/s.48 1.21
*
**************************************************************************/
/*END CMVC */
/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */
/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* C-language container API sample activity implementation program    */
/*                                                                    */
/* Maximum nesting depth is MAX_LEVEL_DEPTH                           */
/* Maximum element number is MAX_LEAF_COUNT                           */
/*                                                                    */
/**********************************************************************/

#define MAX_LEAF_COUNT  200
#define MAX_LEVEL_DEPTH  10


#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fmcjcrun.h>

int main (int argc, char ** argv)
{
  FILE                          *  file1;
  char                             systemGroupName[33] ="";
  char                             systemName[33]      ="";
  char                             user[49]            ="";

  APIRET                           rc                      = FMC_OK;
  FmcjExecutionServiceHandle       service                 = 0;
  FmcjResultHandle                 result                  = 0;
  FmcjReadOnlyContainerHandle      hdlinctnr               = 0;
  FmcjReadWriteContainerHandle     hdloutctnr              = 0;
  FmcjContainerElementVectorHandle members[MAX_LEVEL_DEPTH];
  FmcjContainerElementVectorHandle leaves[MAX_LEAF_COUNT];
  FmcjContainerElementHandle       elementHandle           = 0;
  unsigned long                    ulLeafCount             = 0;
  unsigned long                    ulMemberCount           = 0;
  char                             membername[256]         ="";
  char                             membertype[30]          ="";
  char                             container[4097]         ="";
  char                           * strvalbuf      = &container[0];
  char                             tInfo[4096+1]                            = "";
  long                             lvalbuf        = 0;
  double                           fvalbuf        = 0.0;
  unsigned long                    j              = 0;
  unsigned long                    index          = 0;
  char                           * indexstring    = 0;
  time_t                           ltime          = 0;
  unsigned long                    level, start, current = 0;
  bool                             AllLeavesReached=false;

  file1 = fopen ("fmctjcim.log", "a");

  if ( file1 == 0 )
    return 1;
  time(&ltime);
  fprintf(file1, "\n-----------------------------------------------------\n");
  fprintf(file1, "%s %s", "C-API Activity TestProgram FMCTJCIM called: ", ctime(&ltime));
  fflush(file1);

/*----- Parameter Checking ---------------------------------------------*/
/* t.b.d.                                                               */


/*----- Init API environment -------------------------------------------*/
  rc = FmcjGlobalConnect();
  if (rc != FMC_OK)
  {
    fprintf(file1, "API could not be initialized: %u%\n",rc);
    return -1;
  }


/*----- Provide for detailed error checking ----------------------------*/
  result = FmcjResultObjectOfCurrentThread();
  rc = FmcjResultRc(result);
  if (rc != FMC_OK)
  {
    fprintf(file1, "Result object could not be accessed: %u%\n",rc);
    return -1;
  }
  fprintf(file1, "Accessed result object\n");


/*----- Establish a Passthrough session --------------------------------*/
  rc= FmcjExecutionServiceAllocate( &service );
  if (rc != FMC_OK)
  {
    fprintf(file1, "Service object could not be allocated: %u%\n",rc);
    return -1;
  }

  rc= FmcjExecutionServicePassthrough( service );
  if (rc != FMC_OK)
  {
    fprintf(file1, "Passthrough failed - rc    : %u%\n",rc);
    memset(tInfo,0x00,4097);
  }
  else
  {
                                       /* use service accessors        */
    memset(systemGroupName,0x00,33);
    memset(systemName,0x00,33);
    memset(user,0x00,49);
    FmcjExecutionServiceSystemGroupName(service,systemGroupName,33);
    FmcjExecutionServiceSystemName(service,systemName,33);
    FmcjExecutionServiceUserID(service,user,49);

    fprintf(file1,  "Logged on via Passthrough with user %s on system %s at system group %s\n",
            user, systemName, systemGroupName );
  }
  fflush(file1);


/************************************************************************/
/* First we cope with the activity's INPUT container:                   */
/*  - get the container                                                 */
/*  - get the leaves of the container                                   */
/*  - display the member values                                         */
/************************************************************************/

/*----- Get Input Container --------------------------------------------*/
  rc = FmcjContainerInContainer( &hdlinctnr );
  fprintf(file1, "\nGet Input Container - rc       =  %u \n", rc);
  fflush(file1);
  if (rc != FMC_OK)
  {
    fclose(file1);
    return 1;
  }

  fprintf(file1, "Input Container Name           =  %s \n",
          FmcjReadOnlyContainerType(hdlinctnr, strvalbuf,
                                    FMC_CONTAINER_NAME_LENGTH));

/*----- Get Leave Count ------------------------------------------------*/
  ulLeafCount = FmcjReadOnlyContainerAllLeafCount( hdlinctnr );
  fprintf(file1, "Input Container AllLeafCount() =  %u \n", ulLeafCount);
  fflush(file1);

/*----- Get the Leaves -------------------------------------------------*/
  leaves[0] = FmcjReadOnlyContainerAllLeaves(hdlinctnr);
  if (FmcjResultRc(result) != FMC_OK)
  {
    fprintf(file1, "FmcjContainerAllLeaves returns rc: %u \n", FmcjResultRc(result));
  }

  if (ulLeafCount != FmcjContainerElementVectorSize(leaves[0]))
  {
    fprintf(file1, "LeafCount - Vector Size mismatch: %u %u \n", ulLeafCount,
                   FmcjContainerElementVectorSize(leaves[0]) );
    fflush(file1);
  }

/*----- Show the data members ------------------------------------------*/
  elementHandle = FmcjContainerElementVectorFirstElement(leaves[0]);
  if (FmcjResultRc(result) != FMC_OK)
  {
    fprintf(file1, "FmcjContainerElementVectorFirstElement returns rc: %u \n",
                   FmcjResultRc(result));
  }

  fprintf(file1, "Input Container Leaves:\n");
  fprintf(file1, "\nFull Name                - Type       - Value: \n");
  for (j=0; j < ulLeafCount; j++)
  {
    memset (strvalbuf, 0, 4097);
    strcpy(membername,FmcjContainerElementFullName( elementHandle, strvalbuf, 4096));
    strcpy(membertype,FmcjContainerElementType( elementHandle, strvalbuf, 4096));

    if (strcmp(membertype, "STRING") == 0)
    {
      rc = FmcjContainerElementStringValue(elementHandle,strvalbuf,4096);
      if ( rc != FMC_OK )
      {
        fprintf(file1, "FmcjContainerElementStringValue   %30s %s %u \n",
                       membername, "returns rc: ", rc);
      }
      else
      {
        fprintf(file1, "%20s %10s %20s \n",
                        membername, membertype, strvalbuf);
      }
    }

    if (strcmp(membertype, "LONG") == 0)
    {
      rc = FmcjContainerElementLongValue(elementHandle,&lvalbuf);
      if ( rc != FMC_OK )
      {
        fprintf(file1, "FmcjContainerElementLongValue  %30s %s %u \n",
                       membername, "returns rc: ", rc);
      }
      else
      {
        fprintf(file1, "%20s %10s %20d \n",
                        membername, membertype, lvalbuf);
      }
    }

    if (strcmp(membertype, "FLOAT") == 0)
    {
      rc = FmcjContainerElementFloatValue(elementHandle,&fvalbuf);
      if ( rc != FMC_OK )
      {
        fprintf(file1, "FmcjContainerElementFloatValue %30s %s %u \n",
                       membername, "returns rc: ", rc);
      }
      else
      {
        fprintf(file1, "%20s %10s %20g \n",
                        membername, membertype, fvalbuf);
      }
    }

    if (strcmp(membertype, "BINARY") == 0)
    {
      rc = FmcjContainerElementBinaryValue(elementHandle,strvalbuf,4096);
      if ( rc != FMC_OK )
      {
        fprintf(file1, "FmcjContainerElementBinaryValue%30s %s %u \n",
                       membername, "returns rc: ", rc);
      }
      else
      {
        /* Binary values may contain 0x'00' or control characters     */
        /* Do not try to print!                                       */
        fprintf(file1, "%20s %10s %20s \n",
                        membername, membertype, "???????");
      }
    }

    fflush(file1);
    rc = FmcjContainerElementDeallocate(&elementHandle);
    elementHandle = FmcjContainerElementVectorNextElement(leaves[0]);
  }
  rc= FmcjContainerElementVectorDeallocate(&leaves[0]);
  rc= FmcjReadOnlyContainerDeallocate( &hdlinctnr );

/************************************************************************/
/* Now we process the activity's OUTPUT container:                      */
/*  - get the container                                                 */
/*  - navigate through the ContainerElement levels to the leaves        */
/*  - modify the member values                                          */
/*  - send the container to the server                                  */
/************************************************************************/

/*----- Get Output Container -------------------------------------------*/

  rc= FmcjContainerOutContainer( &hdloutctnr );
  fprintf(file1, "\nGet Output Container - rc   =  %u \n",rc);
  fflush(file1);
  if (rc != FMC_OK)
  {
    fclose(file1);
    return 1;
  }

  fprintf(file1, "Output Container Name       =  %s \n",
          FmcjReadWriteContainerType(hdloutctnr, strvalbuf,
                                     FMC_CONTAINER_NAME_LENGTH));

/*----- Navigate through the ContainerElement structures ---------------*/
  level            = 1;   /* current nesting level                      */
  start            = 0;   /* start "stack pointer" of vectors           */
  current          = 0;   /* end "stack pointer" of vectors             */
  AllLeavesReached = false;

  /* Get the number of 1st level container elements */
  ulMemberCount = FmcjReadWriteContainerMemberCount(hdloutctnr);

  if (ulMemberCount == 0)    /* no data structure at all, error? */
    AllLeavesReached = true;
  else
    /* Get the vector of 1st level container elements */
    members[level] = FmcjReadWriteContainerStructMembers(hdloutctnr);

  while (AllLeavesReached == false)
  {

    fprintf(file1, "\nNumber of members: %u", ulMemberCount);
    fprintf(file1, "\nMember                      -isArray-cardinality-isStruct-isLeaf \n");

    /* Just to check consistency */
    if (FmcjContainerElementVectorSize(members[level]) != ulMemberCount)
    {
      fprintf(file1, "\nFmcjContainerElementVectorSize() - FmcjReadWriteContainerMemberCount() mismatch! %u %u \n", FmcjContainerElementVectorSize(members[level]), ulMemberCount);
    }
    elementHandle = FmcjContainerElementVectorFirstElement(members[level]);
    for (j=0; j < ulMemberCount; j++)
    {
      memset(strvalbuf,0x00,4097);
      fprintf(file1, "%25s",FmcjContainerElementFullName(elementHandle,strvalbuf,4097) );
      fprintf(file1, "   -   " );
      fprintf(file1, "%1u", FmcjContainerElementIsArray(elementHandle) );
      fprintf(file1, "   -   " );
      fprintf(file1, "%3u", FmcjContainerElementCardinality(elementHandle) );
      fprintf(file1, "     -     " );
      fprintf(file1, "%1u", FmcjContainerElementIsStruct(elementHandle) );
      fprintf(file1, "    - " );
      if (FmcjContainerElementIsStruct(elementHandle))
      {
        /* put the next nesting level of this data structure "on the stack" */
        leaves[current] = FmcjContainerElementStructMembers(elementHandle);
        current++;
      }
      fprintf(file1, "%1u\n", FmcjContainerElementIsLeaf(elementHandle) );
      fflush(file1);

      rc = FmcjContainerElementDeallocate(&elementHandle);
      elementHandle = FmcjContainerElementVectorNextElement(members[level]);
    } /* end for */

    if (start >= current)        /* nothing "on the stack" */
    {
      AllLeavesReached = true;
      FmcjContainerElementVectorDeallocate(&members[level]);
    }
    else
    {
      FmcjContainerElementVectorDeallocate(&members[level]);
      level++;
      /* get the next vector from the stack and increase stack pointer */
      members[level] = leaves[start];
      start++;
      ulMemberCount = FmcjContainerElementVectorSize(members[level]);
    }

  } /* end while */

/*----- Modify the data members ----------------------------------------*/
  fprintf(file1, "\n\nSetting the data members... \n");



  leaves[0] = FmcjReadWriteContainerLeaves(hdloutctnr);
  if (FmcjResultRc(result) != FMC_OK)
  {
    fprintf(file1, "FmcjContainerLeaves returns rc: %u \n", FmcjResultRc(result));
  }
  ulLeafCount = FmcjContainerElementVectorSize(leaves[0]);

  elementHandle = FmcjContainerElementVectorFirstElement(leaves[0]);
  if (FmcjResultRc(result) != FMC_OK)
  {
    fprintf(file1, "FmcjContainerElementVectorFirstElement returns rc: %u \n",
                   FmcjResultRc(result));
  }

  for (j=0; j < ulLeafCount; j++)
  {
    memset (strvalbuf, 0, 4097);
    strcpy(membername,FmcjContainerElementFullName( elementHandle, strvalbuf, 4096));
    strcpy(membertype,FmcjContainerElementType( elementHandle, strvalbuf, 4096));

    if (strcmp(membertype, "STRING") == 0)
    {
      if (membername[strlen(membername)-1] == ']')
      /* array element, take the array function */
      {
        memset(&membername[strlen(membername)-1], 0, 1);
        indexstring = strrchr(membername, '[');
        if (indexstring)
        {
          index = atoi( indexstring + 1 );
          memset(indexstring, 0, 1);
          rc= FmcjReadWriteContainerSetArrayStringValue(hdloutctnr,membername,index,"This is a string array value");
          if (rc != FMC_OK)
            fprintf(file1, "\nSetting string value returns rc                  : %u \n", rc);
          rc= FmcjReadWriteContainerArrayStringValue(hdloutctnr,membername,index,strvalbuf,4097);
          fprintf(file1, "- FmcjReadWriteContainerArrayStringValue %22s %s %d %s %20s %s %u \n", membername, "[",index,"] = ",strvalbuf, " rc = ", rc);
        }
      }
      else
      {
        rc= FmcjReadWriteContainerSetStringValue(hdloutctnr,membername,"This is a string value");
        if (rc != FMC_OK)
          fprintf(file1, "\nSetting string value returns rc                  : %u \n", rc);
        rc= FmcjReadWriteContainerStringValue(hdloutctnr,membername,strvalbuf,4097);
        fprintf(file1, "- FmcjReadWriteContainerStringValue %22s %s %20s %s %u \n", membername, "= ",strvalbuf, " rc = ", rc);
      }
    }

    if (strcmp(membertype, "LONG") == 0)
    {
      if (membername[strlen(membername)-1] == ']')
      /* array element, take the array function */
      {
        memset(&membername[strlen(membername)-1], 0, 1);
        indexstring = strrchr(membername, '[');
        if (indexstring)
        {
          index = atoi( indexstring + 1 );
          memset(indexstring, 0, 1);
          rc= FmcjReadWriteContainerSetArrayLongValue(hdloutctnr,membername,index,-42);
          if (rc != FMC_OK)
            fprintf(file1, "\nSetting long value returns rc                    : %u \n", rc);
          rc= FmcjReadWriteContainerArrayLongValue(hdloutctnr,membername,index,&lvalbuf);
          fprintf(file1, "- FmcjReadWriteContainerArrayLongValue %22s %s %d %s %20d %s %u  \n", membername,"[",index,"] = ",lvalbuf, " rc = ", rc);
        }
      }
      else
      {
        rc= FmcjReadWriteContainerSetLongValue(hdloutctnr, membername,42);
        if (rc != FMC_OK)
          fprintf(file1, "\nSetting long value returns rc                    : %u \n", rc);
        rc= FmcjReadWriteContainerLongValue(hdloutctnr,membername,&lvalbuf);
        fprintf(file1, "- FmcjReadWriteContainerLongValue %22s %s %20d %s %u \n", membername, " = ",lvalbuf, " rc = ", rc);
      }
    }

    if (strcmp(membertype, "FLOAT") == 0)
    {
      if (membername[strlen(membername)-1] == ']')
      /* array element, take the array function */
      {
        memset(&membername[strlen(membername)-1], 0, 1);
        indexstring = strrchr(membername, '[');
        if (indexstring)
        {
          index = atoi( indexstring + 1 );
          memset(indexstring, 0, 1);
          rc= FmcjReadWriteContainerSetArrayFloatValue(hdloutctnr,membername,index,-42.42);
          if (rc != FMC_OK)
            fprintf(file1, "\nSetting float value returns rc                   : %u \n", rc);
          rc= FmcjReadWriteContainerArrayFloatValue(hdloutctnr,membername,index,&fvalbuf);
          fprintf(file1, "- FmcjReadWriteContainerArrayFloatValue %22s %s %d %s %20g %s %u  \n", membername,"[",index,"] = ",fvalbuf, " rc = ", rc);
        }
      }
      else
      {
        rc= FmcjReadWriteContainerSetFloatValue(hdloutctnr,membername,42.42);
        if (rc != FMC_OK)
          fprintf(file1, "\nSetting float value returns rc                   : %u \n", rc);
        rc= FmcjReadWriteContainerFloatValue(hdloutctnr,membername,&fvalbuf);
        fprintf(file1, "- FmcjReadWriteContainerFloatValue %22s %s %20g %s %u \n", membername, " = ",fvalbuf, " rc = ", rc);
      }
    }

    if (strcmp(membertype, "BINARY") == 0)
    {
      if (membername[strlen(membername)-1] == ']')
      /* array element, take the array function */
      {
        memset(&membername[strlen(membername)-1], 0, 1);
        indexstring = strrchr(membername, '[');
        if (indexstring)
        {
          index = atoi( indexstring + 1 );
          memset(indexstring, 0, 1);
          rc= FmcjReadWriteContainerSetArrayBinaryValue(hdloutctnr,membername,index,"This is a binary array value", 28);
          if (rc != FMC_OK)
            fprintf(file1, "\nSetting binary value returns rc                  : %u \n", rc);
          rc= FmcjReadWriteContainerArrayBinaryValue(hdloutctnr,membername,index,strvalbuf,28);
          fprintf(file1, "- FmcjReadWriteContainerArrayBinaryValue %20s %s %d %s %20s %s %u \n", membername,"[",index,"] = ","????", " rc = ", rc);
        }
      }
      else
      {
        rc= FmcjReadWriteContainerSetBinaryValue(hdloutctnr,membername,"This is a binary value", 22);
        if (rc != FMC_OK)
          fprintf(file1, "\nSetting binary value returns rc                  : %u \n", rc);
        rc= FmcjReadWriteContainerBinaryValue(hdloutctnr,membername,strvalbuf,22);
        fprintf(file1, "- FmcjReadWriteContainerBinaryValue %20s %s %20s %s %u \n", membername," = ","????", " rc = ", rc);
      }
    }

    fflush(file1);
    rc = FmcjContainerElementDeallocate(&elementHandle);
    elementHandle = FmcjContainerElementVectorNextElement(leaves[0]);
  } /* end for */
  FmcjContainerElementVectorDeallocate(&leaves[0]);

  fflush(file1);

/*----- send the output container to PEA ----------------------------------*/
  rc = FmcjContainerSetOutContainer( hdloutctnr );
  fprintf(file1, "\nSet Output Container - rc   =  %u\n",rc);
  fflush(file1);

  fclose(file1);
  rc= FmcjExecutionServiceLogoff( service );
  rc= FmcjReadWriteContainerDeallocate( &hdloutctnr );
  rc= FmcjExecutionServiceDeallocate( &service );

/*----- Deinit the API environment ---------------------------------------*/
  rc = FmcjGlobalDisconnect();

  return 0;
}


