/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQ Series Workflow API Examples - Check in / check out workitems   */
/*                                                                    */
/* This sample demonstrates how workitems can be checked out of the   */
/* normal MQS workflow environment so that the data contained in the  */
/* input container can be handled externally.                         */
/**********************************************************************/

#include <fmcjcrun.h>                   /* MQWF C API interface       */
#include <time.h>                       /* let's do the time warp again*/
#include <ctype.h>                      /* character classification   */
#include <stdio.h>                      /* for standard in-output     */
#include <stdlib.h>                     /*                            */
#include <string.h>                     /* string conversion          */
#define INFOSIZE 4097



int main (int argc, char ** argv)
{
  FmcjExecutionServiceHandle            service         = 0;
  FmcjResultHandle                      result          = 0;
  FmcjWorkitemVectorHandle              wList           = 0;
  FmcjImplementationDataVectorHandle    impData         = 0;
  FmcjImplementationDataHandle          aImpData        = 0;
  FmcjWorkitemHandle                    workitem        = 0;
  FmcjReadWriteContainerHandle          checkInOutput   = 0;
  FmcjProgramDataHandle                 checkOutData    = 0;
  FmcjReadOnlyContainerHandle           checkOutinctnr  = 0; 
  FmcjReadWriteContainerHandle          checkOutoutctnr = 0;
  FmcjDllOptionsHandle                  dllOptions      = 0;           
  FmcjExeOptionsHandle                  exeOptions      = 0;           
  FmcjExternalOptionsHandle             externalOptions = 0; 
  FmcjErrorHandle                       error           = 0;
  FmcjCDateTime                         timeValue;
  
  unsigned long                         threshold       = 20;
  unsigned long                         vectSize        = 0;
  
  APIRET                                rc              = FMC_OK;

  char                                  user[FMC_USER_ID_LENGTH] = "";
  char                                  choice[1]       = "";
  char                                  tInfo[INFOSIZE] = "";

  int                                   i               = 0;
  int                                   j               = 0;
  int                                   k               = 0;
  int                                   usrchoice       = 0;

/*--------------------------------------------------------------------*/
/* Check command line parameters, logon to system                     */
/*--------------------------------------------------------------------*/
  if (argc!=3)
  {
    printf("\nUsage: %s %s %s\n\n",argv[0],"<UserID>","<Password>");
    return -1;
  }
  fflush(stdout);
  rc = FmcjGlobalConnect();
  result = FmcjResultObjectOfCurrentThread();

  rc = FmcjExecutionServiceAllocate( &service );
  if (rc != FMC_OK)
  {
    printf("Service object could not be allocated: %u\n",rc);
    FmcjGlobalDisconnect();
    return -1;
  }
  /* save argv[1] because it seems to disappear in a block */
  strcpy(user, argv[1]); 
  rc = FmcjExecutionServiceLogon(service, argv[1], argv[2],
                                 Fmc_SM_Default,Fmc_SA_NotSet );
  if (rc != FMC_OK)
  {
    printf("Logon failed with RC  %u %s %s\n",rc," - ",
                      FmcjResultMessageText(result, tInfo, INFOSIZE));
    fflush(stdout);

    /* Deinit API environment */
    rc = FmcjExecutionServiceDeallocate( &service );
    FmcjGlobalDisconnect();
    return rc;
  }
  printf("\nLogged on with UserID '%s' ",
             FmcjExecutionServiceUserID(service, tInfo, INFOSIZE));
  printf("on system '%s' ",
             FmcjExecutionServiceSystemName(service, tInfo, INFOSIZE));
  printf("at system group '%s'.\n",
             FmcjExecutionServiceSystemGroupName(service, tInfo, INFOSIZE));
  fflush(stdout);

/*-------------------------------------------------------------------*/
/* Display a list and prompt user for action                         */
/*-------------------------------------------------------------------*/
  
  do     /* Beginning of main tool loop */
  {
    
    /* Set filter criteria: only those workitems which the logged-on */
    /* user owns will be displayed.                                  */

    rc = FmcjExecutionServiceQueryWorkitems( service,
                                 "OWNER=CURRENT_USER",
                                 "NAME ASC",
                                 &threshold,
                                 &wList );

    if (rc != FMC_OK)
    {
      printf("Querying workitems failed with RC %u-%s\n",
             rc, FmcjResultMessageText(result, tInfo, INFOSIZE));
      fflush(stdout);

      rc = FmcjWorkitemVectorDeallocate(&wList);
      FmcjExecutionServiceLogoff(service);
      /* Deinit API environment */
      rc= FmcjExecutionServiceDeallocate( &service );
      FmcjGlobalDisconnect();
      return -1;
    }

    /* Display as many workitems as specified with threshold */

    printf("\n #  State          Workitem name\n");
    printf("--------------------------------\n");
    fflush(stdout);

    vectSize = FmcjWorkitemVectorSize(wList);
    
    if (vectSize > 0 )
    {
      for(i=0; i < vectSize; i++ )
      {
        if (i < 10) printf(" %u  ",i);
        else        printf("%u  ",i);

        workitem = FmcjWorkitemVectorNextElement(wList);

        switch (FmcjWorkitemState(workitem))
        {
          case Fmc_IS_NotSet:       printf("undefined    "); break;
          case Fmc_IS_Ready:        printf("ready        "); break;
          case Fmc_IS_Finished:     printf("finished     "); break;
          case Fmc_IS_Suspended:    printf("suspended    "); break;
          case Fmc_IS_CheckedOut:   printf("checkedOut   "); break;
          case Fmc_IS_Executed:     printf("executed     "); break;
          case Fmc_IS_ForceFinished:printf("forceFinished"); break;
          case Fmc_IS_Terminating:  printf("terminating  "); break;
          case Fmc_IS_Running:      printf("running      "); break;
          case Fmc_IS_Terminated:   printf("terminated   "); break;
          case Fmc_IS_Disabled:     printf("disabled     "); break;
          case Fmc_IS_InError:      printf("inError      "); break;
          case Fmc_IS_Planning:     printf("planning     "); break;
          case Fmc_IS_Deleted:      printf("deleted      "); break;
          case Fmc_IS_Suspending:   printf("suspending   "); break;
        }
        printf("  %s\n",FmcjWorkitemName(workitem,tInfo,INFOSIZE));
        fflush(stdout);

        rc = FmcjWorkitemDeallocate(&workitem);
      }
    }
    else 
    {
      printf("   <no workitems found for user '%s'>\n", user);
    }

    /* Show actions which user can perform and query user */

    printf("\nCommands: [o] Check out workitem            [v] View workitem info\n");
    printf("          [i] Check in workitem             [q] Quit program\n");
    printf("[o/i/v/q]? ");
    fflush(stdout);


    fgets(choice,3,stdin);

    usrchoice=0;

    if (strncmp(choice,"o",1)==0) usrchoice=1;
    if (strncmp(choice,"i",1)==0) usrchoice=2;
    if (strncmp(choice,"v",1)==0) usrchoice=3;
    if (strncmp(choice,"q",1)==0) usrchoice=255;


    if (vectSize > 0)
    switch (usrchoice)
    {
     case 1:
/*-------------------------------------------------------------------*/
/* Check out a workitem.                                             */
/* In order to do that, the associated process instance must be in   */
/* the RUNNING state, the workitem must be in the READY state and    */
/* must be implemented by a program. In this example, the            */
/* implementation data for the Windows NT platform are queried.      */
/*-------------------------------------------------------------------*/
       do
       {
         if ((vectSize-1) < 100)
           printf("Please enter workitem number (0-%u%s",vectSize-1,") ");
         else
           printf("Please enter workitem number (1-99)");
         fflush(stdout);

         scanf("%2[^\n]",tInfo);
         j=0;
         for (i=0; i < strlen(tInfo); i++)
         { if (isdigit(tInfo[i])) j++; }
         if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
       } while (i!=j);
       i = atoi(tInfo);
       fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
       printf("\n");
       fflush(stdout);
       if (i > (vectSize-1))
       {
         printf("Please use range 0-%u%s\n",vectSize-1,".");
       }
       else    /* user's choice is in range */
       {
         /* Get the selected workitem */
         workitem = FmcjWorkitemVectorFirstElement(wList);
         for (j = 0; j < i; j++) 
         {
           rc = FmcjWorkitemDeallocate(&workitem);
           workitem = FmcjWorkitemVectorNextElement(wList);
         }
       
         rc = FmcjWorkitemCheckOut(workitem,
                                 Fmc_WS_SpecifiedDefinitions,
                                 Fmc_DP_WindowsNT,
                                 &checkOutData );
         if ( rc!=FMC_OK )
         {
            printf("Checking out workitem failed with RC %u-%s\n\n", rc,
                    FmcjResultMessageText(result, tInfo, INFOSIZE));
            fflush(stdout);         }
         else
         {
           /* Now it's possible to get the input and output container */
           /* from the FmcjProgramData object                         */
           
           checkOutinctnr  = FmcjProgramDataInContainer(checkOutData);
           checkOutoutctnr = FmcjProgramDataOutContainer(checkOutData);
           
           impData = FmcjProgramDataImplementations(checkOutData);
           
           printf("Workitem has been checked out.\n");
           printf("Input Container name : ");
           printf("\n", FmcjContainerType(checkOutinctnr, tInfo, INFOSIZE));
           printf("Output Container name: ");
           printf("\n", FmcjContainerType(checkOutoutctnr, tInfo, INFOSIZE));
           printf("Execution user       : ");

           switch(FmcjProgramDataExecutionUser(checkOutData))
           {
             case Fmc_GU_NotSet:       printf("Not Set"); break; 
             case Fmc_GU_Agent:        printf("Agent  "); break; 
             case Fmc_GU_Starter:      printf("Starter"); break; 
           }
           rc = FmcjReadOnlyContainerDeallocate(&checkOutinctnr);
           rc = FmcjReadWriteContainerDeallocate(&checkOutoutctnr);


           printf("\n\n");
           printf("Windows NT Implementation Data\n");
           printf("------------------------------\n");

           fflush(stdout);

           aImpData = FmcjImplementationDataVectorFirstElement(impData);
           
           switch (FmcjImplementationDataKind(aImpData))
           {
            case Fmc_DT_DLL:

              /* Fetch DLL implementation data */
              
              dllOptions = FmcjImplementationDataDllOptions(aImpData);

              printf("Path and file name: %s\n", 
                     FmcjDllOptionsPathAndFileName(
                                dllOptions, tInfo, INFOSIZE));
                     
              printf("Entry point name  : %s\n",
                     FmcjDllOptionsEntryPointName(
                                dllOptions, tInfo, INFOSIZE));
                     
              fflush(stdout);
              rc = FmcjDllOptionsDeallocate(&dllOptions);
              break;

            case Fmc_DT_EXE:

              /* Fetch EXE implementation data */

              exeOptions = FmcjImplementationDataExeOptions(aImpData);
              
              printf("Path and file name: %s\n", 
                     FmcjExeOptionsPathAndFileName(
                                exeOptions, tInfo, INFOSIZE));
                     
              printf("Working directory : %s\n", 
                     FmcjExeOptionsWorkingDirectoryName(
                                exeOptions, tInfo, INFOSIZE));
              
              fflush(stdout);
              rc = FmcjExeOptionsDeallocate(&exeOptions);
              break;

            case Fmc_DT_External:

              /* Fetch external implementation data */

              externalOptions=FmcjImplementationDataExternalOptions(aImpData);
              
              printf("Executable name: %s\n", 
                     FmcjExternalOptionsExecutableName(
                                externalOptions, tInfo, INFOSIZE));
              
              printf("Invocation type: %s\n", 
                     FmcjExternalOptionsInvocationType(
                                externalOptions, tInfo, INFOSIZE));
              
              fflush(stdout);
              rc = FmcjExternalOptionsDeallocate(&externalOptions);
              break;
           }
           rc = FmcjImplementationDataVectorDeallocate(&impData);
           rc = FmcjImplementationDataDeallocate(&aImpData);
         }
         rc = FmcjWorkitemDeallocate(&workitem);
       }

       break;


     case 2:
/*-------------------------------------------------------------------*/
/* Check in a workitem                                               */
/*-------------------------------------------------------------------*/
       do
       {
         if ((vectSize-1) < 100)
           printf("Please enter workitem number (0-%u%s",vectSize-1,") ");
         else
           printf("Please enter workitem number (1-99)");
         fflush(stdout);

         scanf("%2[^\n]",tInfo);
         j=0;
         for (i=0; i < strlen(tInfo); i++)
         { if (isdigit(tInfo[i])) j++; }
         if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
       } while (i!=j);
       i = atoi(tInfo);
       fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
       printf("\n");
       fflush(stdout);
       if (i > (vectSize-1))
       {
         printf("Please use range 0-%u%s\n",vectSize-1,".");
       }
       else    /* user's choice is in range */
       {
         /* Get the selected workitem */
         workitem = FmcjWorkitemVectorFirstElement(wList);
         for (j = 0; j < i; j++) 
         {
           rc = FmcjWorkitemDeallocate(&workitem);
           workitem = FmcjWorkitemVectorNextElement(wList);
         }

         do
         {
           printf("Enter desired RC of workitem: ");
           fflush(stdout);

           scanf("%2[^\n]",tInfo);
           j=0;
           for (k=0; k < strlen(tInfo); k++)
           { if (isdigit(tInfo[i])) j++; }
           if (k!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
         } while (k!=j);
         j = atoi(tInfo);
         fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
         printf("\n");
         fflush(stdout);

         /* Get ReadWriteContainer from workitem */
         rc = FmcjWorkitemOutContainer(workitem, &checkInOutput);
          
         if (rc != FMC_OK)
         {
           printf("Getting Output Container from workitem failed with RC %u-%s\n\n", 
                  rc, FmcjResultMessageText(result, tInfo, INFOSIZE));
           fflush(stdout);
         }
         
         /* Check in workitem */
         rc = FmcjWorkitemCheckIn(workitem, checkInOutput, j );
         if ( rc!=FMC_OK )
         {
           printf("Checking in workitem failed with RC %u-%s\n\n", 
                  rc, FmcjResultMessageText(result, tInfo, INFOSIZE));
           fflush(stdout);
         }
         
         rc = FmcjWorkitemDeallocate(&workitem);
         rc = FmcjReadWriteContainerDeallocate(&checkInOutput);

       }

       break;


      case 3:
/*-------------------------------------------------------------------*/
/* Show some workitem info                                           */
/*-------------------------------------------------------------------*/
        do
        {
          if ((vectSize-1) < 100)
            printf("Please enter workitem number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter workitem number (1-99)");
          fflush(stdout);
         
          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);
        if (i > (vectSize-1))
        {
          printf("Please use range 0-%u%s\n",vectSize-1,".");
        }
        else    /* user's choice is in range */
        {
          /* Get the selected workitem */
          workitem = FmcjWorkitemVectorFirstElement(wList);
          for (j = 0; j < i; j++) 
          {
            rc = FmcjWorkitemDeallocate(&workitem);
            workitem = FmcjWorkitemVectorNextElement(wList);
          }

          printf("Workitem name         : %s\n", 
                  FmcjWorkitemName(workitem, tInfo, INFOSIZE));
          printf("Process instance name : %s\n", 
                  FmcjWorkitemProcessInstanceName(workitem, tInfo, INFOSIZE));
          
          timeValue = FmcjWorkitemCreationTime(workitem);
          printf("Creation time         : %s\n", 
                  FmcjDateTimeAsString(&timeValue, tInfo, INFOSIZE));
          printf("Program implementation: %s\n", 
                  FmcjWorkitemImplementation(workitem, tInfo, INFOSIZE));
          
          if (!FmcjWorkitemErrorReasonIsNull(workitem))
          {
            error = FmcjWorkitemErrorReason(workitem);
            printf("\nError RC  : %u\n", FmcjErrorRc(error));
            printf("Error text: %s\n",
                       FmcjErrorMessageText(error, tInfo, INFOSIZE));
            fflush(stdout);
            rc = FmcjErrorDeallocate(&workitem);
          }
          rc = FmcjWorkitemDeallocate(&workitem);
        }

        break;

    }               /* end switch */
    else printf("\nNo workitems available!\n");

    fflush(stdout);
    rc = FmcjWorkitemVectorDeallocate(&wList);

    
  } while ( (usrchoice!=255) );


/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/

  printf("\nLogging off.\n");
  rc = FmcjExecutionServiceLogoff(service);

  rc= FmcjExecutionServiceDeallocate( &service );

  printf("Deinitializing API environment.\n");
  rc = FmcjGlobalDisconnect();


  return 0;
}
