/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQSeries Workflow API Examples - Handling process instances        */
/*                                                                    */
/* This example shows what can be done with process instances queried */
/* from the execution service.                                        */
/**********************************************************************/

#include <fmcjcrun.h>                   /* MQWF C API interface       */
#include <ctype.h>                      /* character classification   */
#include <time.h>                       /* let's do the time warp again*/
#include <stdlib.h>                     /*                            */
#include <string.h>                     /* string conversion          */
#include <stdio.h>                      /* for standard in-output     */
#define INFOSIZE 4097

int main (int argc, char ** argv)
{

  FmcjExecutionServiceHandle        service     = 0;
  FmcjProcessInstanceVectorHandle   iList       = 0;
  FmcjProcessInstanceHandle         instance    = 0;
  FmcjCDateTime                     timeValue;
  FmcjResultHandle                  result      = 0;
  FmcjReadWriteContainerHandle      inContainer = 0;

  unsigned short                    year        = 0;
  unsigned short                    month       = 0;
  unsigned short                    day         = 0;
  unsigned short                    hour        = 0;
  unsigned short                    minute      = 0;
  
  unsigned long                     threshold   = 20;
  unsigned long                     i           = 0;
  unsigned long                     j           = 0;
  unsigned long                     vectSize    = 0;

  APIRET                            rc          = FMC_OK;

  int                               usrchoice   = 0;
  
  time_t                            ltime       = 0;

  char                              choice[1]         = "";
  char                              tInfo[INFOSIZE]   = "";


/*--------------------------------------------------------------------*/
/* Check command line parameters, logon to system                     */
/*--------------------------------------------------------------------*/

  if (argc!=3)
  {
    printf("\nUsage: %s %s %s\n\n",argv[0],"<UserID>","<Password>");
    return -1;
  }
  fflush(stdout);

  rc = FmcjGlobalConnect();
  result = FmcjResultObjectOfCurrentThread();

  rc= FmcjExecutionServiceAllocate( &service );
  if (rc != FMC_OK)
  {
    printf("Service object could not be allocated: %u\n",rc);
    FmcjGlobalDisconnect();
    return -1;
  }

  rc = FmcjExecutionServiceLogon(service, argv[1], argv[2],
                                 Fmc_SM_Default,Fmc_SA_NotSet );
  if (rc != FMC_OK)
  {

    printf("Logon failed with RC  %u %s %s\n",rc," - ",
                    FmcjResultMessageText(result, tInfo, INFOSIZE));
    fflush(stdout);

    /* Deinit API environment */
    rc= FmcjExecutionServiceDeallocate( &service );
    FmcjGlobalDisconnect();
    return -1;
  }

  printf("Logged on with UserID '%s'\n",
             FmcjExecutionServiceUserID(service, tInfo, INFOSIZE));
  printf("on system '%s'\n",
             FmcjExecutionServiceSystemName(service, tInfo, INFOSIZE));
  printf("at system group '%s'.\n",
             FmcjExecutionServiceSystemGroupName(service, tInfo, INFOSIZE));
  fflush(stdout);


/*--------------------------------------------------------------------*/
/* Display current instances list, query user for action              */
/*--------------------------------------------------------------------*/

  do /* Beginning of main tool loop*/
  {
     /* Display as many process instances as specified with threshold */

    printf("\nAvailable process instances:\n\n");

    rc = FmcjExecutionServiceQueryProcessInstances(service,
                                                   FmcjNoFilter,
                                                   "NAME ASC",
                                                   &threshold,
                                                   &iList );

    if (rc != FMC_OK)
    {

      FmcjResultMessageText(result, tInfo, INFOSIZE);
      printf("Querying process instances failed with RC %u%s%s\n",
             rc," - ",FmcjResultMessageText(result, tInfo, INFOSIZE));
      fflush(stdout);

      FmcjProcessInstanceVectorDeallocate(&iList);
      FmcjExecutionServiceLogoff(service);
      /* Deinit API environment */
      rc= FmcjExecutionServiceDeallocate( &service );
      FmcjGlobalDisconnect();
      return -1;
    }

    printf(" #  State        Instance name \n" );
    printf("------------------------------ \n" );
    fflush(stdout);

    vectSize = FmcjProcessInstanceVectorSize(iList);
    if (vectSize > 0)
    {
      for( i=0; i< vectSize; i++ )
      {
        if (i < 10) printf(" %u%s",i,"  ");
        else        printf("%u%s",i,"  ");

        instance = FmcjProcessInstanceVectorNextElement(iList);
                
        switch (FmcjProcessInstanceState(instance))
        {
         case Fmc_PS_NotSet:      printf("undefined   "); break;
         case Fmc_PS_Ready:       printf("ready       "); break;
         case Fmc_PS_Running:     printf("running     "); break;
         case Fmc_PS_Finished:    printf("finished    "); break;
         case Fmc_PS_Terminated:  printf("terminated  "); break;
         case Fmc_PS_Suspended:   printf("suspended   "); break;
         case Fmc_PS_Terminating: printf("terminating "); break;
         case Fmc_PS_Suspending:  printf("suspending  "); break;
         case Fmc_PS_Deleted:     printf("deleted     "); break;
        }

        printf(" %s\n",FmcjProcessInstanceName(instance,tInfo,INFOSIZE));
        fflush(stdout);

        rc = FmcjProcessInstanceDeallocate(&instance);
      }
    }
    else printf("  <no instances available>\n");

    fflush(stdout);
    printf("\nCommands: [v] View instance information             [t] Terminate instance\n");
    printf("          [s] Suspend instance                      [n] Start instance\n");
    printf("          [r] Resume instance\n");
    printf("          [a] Restart instance                      [q] Quit program\n");
    printf("[v/s/r/a/t/n/q]? ");
    fflush(stdout);

    fgets(choice,3,stdin);

    usrchoice=0;

    if (strncmp(choice,"v",1)==0) usrchoice=1;
    if (strncmp(choice,"s",1)==0) usrchoice=2;
    if (strncmp(choice,"r",1)==0) usrchoice=3;
    if (strncmp(choice,"a",1)==0) usrchoice=4;
    if (strncmp(choice,"t",1)==0) usrchoice=5;
    if (strncmp(choice,"n",1)==0) usrchoice=6;
    if (strncmp(choice,"q",1)==0) usrchoice=255;

    if (vectSize > 0)
    switch (usrchoice)
    {
     case 1:
/*-------------------------------------------------------------------*/
/* Show some instance properties                                     */
/*-------------------------------------------------------------------*/

       do
       {
         if ((vectSize-1) < 100)
           printf("Please enter instance number (0-%u%s",vectSize-1,") ");
         else
           printf("Please enter instance number (1-99)");
         fflush(stdout);
         
         scanf("%2[^\n]",tInfo);
         j=0;
         for (i=0; i < strlen(tInfo); i++)
         { if (isdigit(tInfo[i])) j++; }
         if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
       } while (i!=j);
       i = atoi(tInfo);
       fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
       printf("\n");
       fflush(stdout);
       if (i > (vectSize-1))
       {
         printf("Please use range 0-%u%s\n",vectSize-1,".");
       }
       else    /* user's choice is in range */
       {
         /* Get the selected instance */
         instance = FmcjProcessInstanceVectorFirstElement(iList);
         for (j = 0; j < i; j++) 
         {
           rc = FmcjProcessInstanceDeallocate(&instance);
           instance = FmcjProcessInstanceVectorNextElement(iList);
         }
         
         printf("Instance properties\n");
         printf("------------------- \n");
         printf("   Name     : %s\n",
           FmcjProcessInstanceName(instance, tInfo, INFOSIZE));
         printf("   Object id: %s\n",
           FmcjProcessInstancePersistentOid(instance, tInfo, INFOSIZE));
         timeValue = FmcjProcessInstanceSuspensionExpirationTime(instance);
         printf("   Suspension expiration time: %s\n\n",
           FmcjDateTimeAsString(&timeValue, tInfo, INFOSIZE));

         rc = FmcjProcessInstanceDeallocate(&instance);
       }
       fflush(stdout);
       break;


      case 2:
/*-------------------------------------------------------------------*/
/* Demonstrates how to suspend a process instance.                   */
/*-------------------------------------------------------------------*/

        do
        {
          if ((vectSize-1) < 100)
            printf("Please enter instance number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter instance number (1-99)");
          fflush(stdout);
         
          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);        if (i > (vectSize-1))
        {
          printf("Please use range 0-%u%s\n",vectSize-1,".");
        }
        else    /* user's choice is in range */
        {
          /* Get the selected instance */
          instance = FmcjProcessInstanceVectorFirstElement(iList);
          for (j = 0; j < i; j++) 
          {
            rc = FmcjProcessInstanceDeallocate(&instance);
            instance = FmcjProcessInstanceVectorNextElement(iList);
          }
          
          time(&ltime);
          printf("Current time: %s",ctime(&ltime));
          printf("\nDo you want to set a suspension expiration time? [y/n] ");
          fflush(stdout);

          fgets(choice,3,stdin);

          if (strncmp(choice,"y",1)==0)
          {
            printf("\nEnter year  : "); fflush(stdout); fgets(tInfo,6,stdin);
            year = atoi(tInfo);
            printf("\nEnter month : "); fflush(stdout); fgets(tInfo,4,stdin);
            month = atoi(tInfo);
            printf("\nEnter day   : "); fflush(stdout); fgets(tInfo,4,stdin);
            day = atoi(tInfo);
            printf("\nEnter hour  : "); fflush(stdout); fgets(tInfo,4,stdin);
            hour = atoi(tInfo);
            printf("\nEnter minute  : "); fflush(stdout);fgets(tInfo,4,stdin);
            minute = atoi(tInfo);

            timeValue.year  = year;
            timeValue.month = month;
            timeValue.day   = day;
            timeValue.hour  = hour;
            timeValue.minute= minute;
            
            /* Suspend the process instance and all subordinated processes */
            /* and let them be resumed automatically at the specified time.*/
            rc = FmcjProcessInstanceSuspendUntil(instance, &timeValue, true );

          }
          else
          {
            /* Suspend the process instance and all subordinated processes */

            rc = FmcjProcessInstanceSuspend(instance, true );           }

          if (rc != FMC_OK)
          {
            printf("\nSuspending instance failed with RC %u-%s\n\n", rc,
                    FmcjResultMessageText(result, tInfo, INFOSIZE));
            fflush(stdout);
          }
          rc = FmcjProcessInstanceDeallocate(&instance);
        } /* end else: user's choice is in range */
        break;


      case 3:
/*-------------------------------------------------------------------*/
/* Demonstrates how to resume a process instance.                    */
/*-------------------------------------------------------------------*/

        do
        {
          if ((vectSize-1) < 100)
            printf("Please enter instance number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter instance number (1-99)");
          fflush(stdout);
         
          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);
        if (i > (vectSize-1))
        {
          printf("Please use range 0-%u%s\n",vectSize-1,".");
        }
        else    /* user's choice is in range */
        {
          /* Get the selected instance */
          instance = FmcjProcessInstanceVectorFirstElement(iList);
          for (j = 0; j < i; j++) 
          {
            rc = FmcjProcessInstanceDeallocate(&instance);
            instance = FmcjProcessInstanceVectorNextElement(iList);
          }
          
          /* Resume the process instance and all subordinated processes */
          rc = FmcjProcessInstanceResume(instance, true );
          if (rc != FMC_OK)
          {
            printf("\nResuming instance failed with RC %u%s%s\n\n", rc,
                   " - ", FmcjResultMessageText(result, tInfo, INFOSIZE));
            fflush(stdout);
          }
          rc = FmcjProcessInstanceDeallocate(&instance);
        }
        break;


      case 4:
/*-------------------------------------------------------------------*/
/* Demonstrates how to restart a process instance.                   */
/*-------------------------------------------------------------------*/

        do
        {
          if ((vectSize-1) < 100)
            printf("Please enter instance number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter instance number (1-99)");
          fflush(stdout);
         
          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);
        if (i > (vectSize-1))
        {
          printf("Please use range 0-%u%s\n",vectSize-1,".");
        }
        else    /* user's choice is in range */
        {
          /* Get the selected instance */
          instance = FmcjProcessInstanceVectorFirstElement(iList);
          for (j = 0; j < i; j++) 
          {
            rc = FmcjProcessInstanceDeallocate(&instance);
            instance = FmcjProcessInstanceVectorNextElement(iList);
          }
          
          rc = FmcjProcessInstanceRestart(instance);
          if (rc != FMC_OK)
          {
            printf("\nRestarting instance failed with RC %u%s%s\n\n", rc,
                   " - ", FmcjResultMessageText(result, tInfo, INFOSIZE));
            fflush(stdout);
          }
          rc = FmcjProcessInstanceDeallocate(&instance);
        }
        break;


      case 5:
/*-------------------------------------------------------------------*/
/* Demonstrates how to terminate a process instance.                 */
/*-------------------------------------------------------------------*/

        do
        {
          if ((vectSize-1) < 100)
            printf("Please enter instance number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter instance number (1-99)");
          fflush(stdout);
         
          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);
        if (i > (vectSize-1))
        {
          printf("Please use range 0-%u%s\n",vectSize-1,".");
        }
        else    /* user's choice is in range */
        {
          /* Get the selected instance */
          instance = FmcjProcessInstanceVectorFirstElement(iList);
          for (j = 0; j < i; j++) 
          {
            rc = FmcjProcessInstanceDeallocate(&instance);
            instance = FmcjProcessInstanceVectorNextElement(iList);
          }
          
          rc = FmcjProcessInstanceTerminate(instance);
          if (rc != FMC_OK)
          {
            printf("\nTerminating instance failed with RC %u%s%s\n\n", rc,
                   " - ", FmcjResultMessageText(result, tInfo, INFOSIZE));
            fflush(stdout);
          }
          rc = FmcjProcessInstanceDeallocate(&instance);
        }
        break;


      case 6:
/*-------------------------------------------------------------------*/
/* Demonstrates how to start a process instance.                     */
/*-------------------------------------------------------------------*/

        do
        {
          if ((vectSize-1) < 100)
            printf("Please enter instance number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter instance number (1-99)");
          fflush(stdout);
         
          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);
        if (i > (vectSize-1))
        {
          printf("Please use range 0-%u%s\n",vectSize-1,".");
        }
        else    /* user's choice is in range */
        {
          /* Get the selected instance */
          instance = FmcjProcessInstanceVectorFirstElement(iList);
          for (j = 0; j < i; j++) 
          {
            rc = FmcjProcessInstanceDeallocate(&instance);
            instance = FmcjProcessInstanceVectorNextElement(iList);
          }

          /* Start instance without passing an input container */
          rc = FmcjProcessInstanceStart(instance, inContainer);
          if (rc != FMC_OK)
          {
            printf("\nTerminating instance failed with RC %u%s%s\n\n", rc,
                   " - ", FmcjResultMessageText(result, tInfo, INFOSIZE));
            fflush(stdout);
          }
          rc = FmcjProcessInstanceDeallocate(&instance);
        }
        break;

      }                                 /* end of switch */
      else printf("\nNo instances available!\n");
      fflush(stdout);
      FmcjProcessInstanceVectorDeallocate(&iList);

  } while ( (usrchoice!=255) );         /* end of main tool loop */


/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/

  printf("\nLogging off.\n");
  rc = FmcjExecutionServiceLogoff(service);

  rc= FmcjExecutionServiceDeallocate( &service );

  printf("Deinitializing API environment.\n");
  rc = FmcjGlobalDisconnect();

  return 0;
}
