//BEGIN CMVC */
//*************************************************************************
//
//  Workfile: java/exit/Authentication.java, v3-java, fmv3
//  Last update: 02/11/29 17:58:05
//  SCCS path, id: /home/flowmark/vc/0/3/7/0/s.95 1.9
//
//************************************************************************/
//END CMVC */
/*BEGIN COPYRIGHT */
//*************************************************************************
//
//  Licensed Materials - Property of IBM
//  5697-FM3 5655-A96
//  (C) Copyright IBM Corp. 1999, 2001
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
//************************************************************************/
/*END COPYRIGHT */
/*BEGIN DISCLAIMER */
//*************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
//************************************************************************/
/*END DISCLAIMER */
//****************************************************************************
//
//  This is a sample implementation of the Server Authentication Exit. It can
//  handle X509 certificate and credential objects of the Credentials class.
//  In the example no real authentication takes place. The decision to grant
//  or deny authentication is hard coded in the sample code. A real
//  implementation should use other means for authentication like a LDAP
//  directory or something similar.
//
//****************************************************************************
//****************************************************************************
// SETUP
//
// NOTE: It is a known problem, that a MQWF AuthenticationExit written in Java does not work with
//       SUN JDKs 1.3.1 prior to service level 4!
//
// Copy the files/directory below <MQWFDir>\SMP\java\authexit to a directory structure com/ibm/workflow/java/exit 
// and switch to that directory.
// To enable the sample AuthenticationExit you must compile the Authentication.java
// file located in the com\ibm\workflow\java\exit subdirectory below your current directory and
// the Helper classes used (sample/Credentials.java, sample/AuthenticationClient.java), e.g.:
//
// >javac -classpath <MQWFDir>\bin\JAVA<VRMS>\fmcojagt.jar com\ibm\workflow\java\exit\sample\*.java
// >javac -classpath <MQWFDir>\bin\java<VRMS>\fmcojagt.jar;. com\ibm\workflow\java\exit\Authentication.java
//
// It is recommended to create a Java Archive file for the AuthenticationExit, e.g.:
//
// >jar -cvf fmcaexit.jar com
//
// You have to set the environment for the MQWF Administration Server correctly if you want
// to use the Exit:
//
// 1. Set the classpath in the environment the MQWF Administration Server is going to run in,
//    e.g.:
//         set classpath=D:\INFO\MQWF\AuthenticationExit\fmcaexit.jar;%classpath%
//
//    NOTE: The file fmcojagt.jar must be available and part of the CLASSPATH!
//
// 2. Enable the Java AuthenticationExit by executing:
//
//         fmczchk -c inst:m,RTAuthenticationExitTypeServer,JAVA -y <Cfg-ID>
//
//    NOTE: To disable the AuthenticationExit use:
//           fmczchk -c inst:m,RTAuthenticationExitTypeServer -y <Cfg-ID>
//
// 3. Make sure, that a Java 2 Runtime Environment is available in the MQWF Administration Server's
//    environment.
//
// After (re-)starting the MQWF Administration Server the AuthenticationExit will be enabled
//
//
// A sample client program is provided in sample/AuthenticationClient.java that shows the
// new logon methods 'logon3(byte[] userCredentials)' and
//                   'logon4(byte[] userCredentials, SessionMode mode,
//                           AbsenceIndicator absenceIndicator, java.lang.String userName)
// that are used to pass credentials to the AuthenticationExit.
//
// Follow the instructions in sample/readme.txt to create a sample X509 certificate that can be used
// by the sample client. You can find the instructions how to run the sample there as well.
//
//********************************************************************************************************
package com.ibm.workflow.java.exit;

// import needed java packages
import java.util.*;
import java.security.cert.*;
import java.io.*;

// import user defined class
import com.ibm.workflow.java.exit.sample.Credentials;

public class Authentication implements AuthenticationConstants
{
  /**
  * This comment block documents all parameters passed into and out off
  * the Authentication Exit
  */
//public static int authenticate(Hashtable exitParam)
//{
//  // input parameters
//  Integer exitVersion = (Integer) exitParam.get("exitParameterListVersion");
//  Integer version = (Integer) exitParam.get("version");
//  Integer release = (Integer) exitParam.get("release");
//  Integer modLevel = (Integer) exitParam.get("modlevel");
//  byte[] credentials = (byte[]) exitParam.get("userCredentials");
//  String correlID = (String) exitParam.get("exitCorrelID");
//  Integer phase = (Integer) exitParam.get("phaseNumber");
//  String result = (String) exitParam.get("exitResult");
//  Integer reason = (Integer) exitParam.get("reasonCode");
//  String userName = (String) exitParam.get("userName");
//  String mqwfUserID = (String) exitParam.get("mqwfUserID");
//
//  // request processing
//  correlID = "<correlID>";
//  result = FMC_LOGON_ACCEPTED; // FMC_LOGON_DENIED, FMC_ERROR, FMC_NEXT_PHASE (reserved for future use)
//  // The result object is passed back to the MQWF API as return code of a logon3()/logon4() call.
//  reason = new Integer(0);
//  userName = "<userName>";
//  mqwfUserID = "<mqwfUserID>";
//  credentials = "<credentials>".getBytes();
//  int rc = 0; // FMC_EXIT_OK (-> 0),  FMC_EXIT_RECOVERABLE_ERROR (-> 1), FMC_EXIT_NONRECOVERABLE_ERROR (->  2)
//  // The return code is passed back to the MQWF Administration Server.

//
//  // output parameters
//  exitParam.put("exitCorrelID", correlID);
//  exitParam.put("exitResult", result);
//  exitParam.put("reasonCode", reason);
//  exitParam.put("userName", userName);
//  exitParam.put("mqwfUserID", mqwfUserID);
//  exitParam.put("userCredentials", credentials);
//
//  return rc;
//}

  /**
  * method to print the parameters passed to the exit on the screen.
  * Note that the output is printed to the output of the administration server of MQWF!
  *
  * Test implementation
  */
  public static int authenticate(Hashtable exitParam)
  {
    // System.out.println("Exit called!");
    String credentialsUserName ="";
    // first extract information passed to the java Authentication exit
    Integer exitVersion = (Integer) exitParam.get("exitParameterListVersion");
    Integer version = (Integer) exitParam.get("version");
    Integer release = (Integer) exitParam.get("release");
    Integer modLevel = (Integer) exitParam.get("modlevel");
    byte[] credentials = (byte[]) exitParam.get("userCredentials");
    String correlID = (String) exitParam.get("exitCorrelID");
    Integer phase = (Integer) exitParam.get("phaseNumber");
    String result = (String) exitParam.get("exitResult");
    Integer reason = (Integer) exitParam.get("reasonCode");
    String userName = (String) exitParam.get("userName");
    String mqwfUserID = (String) exitParam.get("mqwfUserID");

    // if you like to see the information passed to the exit printed on the screen uncomment
    // the following line!
    //dump(exitParam);
    // ExitParam.dump (11 keys):
    //   reasonCode = .....
    //   modlevel = .....
    //   exitResult = .....
    //   userCredentials = .....
    //   release = .....
    //   exitParameterListVersion = .....
    //   version = .....
    //   exitCorrelID =.....
    //   mqwfUserID =.....
    //   userName = .....
    //   phaseNumber = .....
    byte[] passedCredentials = credentials; // used to save the byte[]'s data;
                                       // ByteArrayInputStream constructor does modify the data!

    // then use the credentials to authenticate the user
    // try if the passed byte array contains a valid X509 certificate
    // this is done by trying to recreate a X509 certificate from the data. If an exception occurrs,
    // the byte[] did not contain a X509 certificate and it is assumed, that the client sent a user
    // defined credential.
    ByteArrayInputStream byteIn = new ByteArrayInputStream(credentials);
    try
    {
      CertificateFactory cf = CertificateFactory.getInstance("X509");
      X509Certificate cert = (X509Certificate) cf.generateCertificate(byteIn);
      // If the passed byte[] contained a valid X509 certificate, the information of the certificate
      // is extracted and used to authenticate the user.
      boolean isValid = false;
      try
      {
        cert.checkValidity();
        isValid = true;
      }
      catch(CertificateExpiredException certExpEx)
      {
        isValid = false;
        System.out.println("Certificate has expired");
        result = FMC_LOGON_DENIED;
        reason = new Integer(21);  //reason codes are user specific, no meaning associated in this example
      }
      catch(CertificateNotYetValidException certNotYetValidEx)
      {
        isValid = false;
        System.out.println("Certificate is not valid yet!");
        result = FMC_LOGON_DENIED;
        reason = new Integer(22);
      }
      if (isValid)
      {
        // The certificate is valid, now check if the user is authorized...
        String certSubject = (cert.getSubjectDN()).getName();
        credentialsUserName = extractCN(certSubject);
        System.out.println(" The credentialsUserName is set to :" + credentialsUserName);
        // ... here the user's credentials should be checked against a LDAP directory or something
        // similar. For simplicity the example uses a hard coded method only, which accepts or denies
        // logon only based on the userName!
        if (checkUserName(credentialsUserName))
        {
          result = FMC_LOGON_ACCEPTED;
          mqwfUserID = mapUserName(credentialsUserName);
          userName = credentialsUserName;
        }
        else
        {
          result = FMC_LOGON_DENIED;
          reason =  new Integer(2);
          mqwfUserID = "";
          userName = "";
        }
      }
    }
    catch(CertificateException certEx)
    {
      // The byte[] did not contain a valid X509 certificate, but probably a user defined byte[]
      // If it is a user defined credential, it was passed as a serialized object, thus it must be
      // deserialized first

      // To avoid sideeffects from the test for X509 certificate data, another InputStream is created.
      ByteArrayInputStream byteInNew = new ByteArrayInputStream(passedCredentials);
      try
      {
        ObjectInputStream in = new ObjectInputStream(byteInNew);
        Credentials myCert = (Credentials) in.readObject();
        in.close();
        credentialsUserName = myCert.getUser();
        if (checkUserName(credentialsUserName))
        {
          result = FMC_LOGON_ACCEPTED;
          mqwfUserID = mapUserName(credentialsUserName);
          userName = credentialsUserName;
        }
        else
        {
          result = FMC_LOGON_DENIED;
          reason = new Integer(2);
          mqwfUserID = "";
          userName = "";
        }

      }
      catch(Exception ex)
      {
        System.out.println("Exception occured deserializing credential");
        ex.printStackTrace();
        result = FMC_ERROR;
        reason = new Integer(91);
      }

    }
    catch(Exception except)
    {
      System.out.println(" Exception occured while extracting certificate");
      except.printStackTrace();
      result = FMC_ERROR;
      reason = new Integer(92);
    }
    // Note: The following output is printed to the output of the MQWF admin server, i.e.
    // you won't see it, if you run the MQWF admin server as a Windows NT service!
    System.out.println("userName: " + userName.toUpperCase());
    System.out.println("mqwfUserID: " + mqwfUserID.toUpperCase());
    System.out.println("reasonCode: " + reason);
    System.out.println("exitResult: " + result);
    exitParam.put("userName", userName.toUpperCase());
    exitParam.put("mqwfUserID", mqwfUserID.toUpperCase());
    exitParam.put("reasonCode", reason);
    exitParam.put("exitResult", result);
    // System.out.println("End of Exit reached!");
    return 0;
  }

  private static void dump(Hashtable exitParam)
  {
    System.out.println("ExitParam.dump (" + exitParam.size() + " keys):");

    Enumeration keys = exitParam.keys();

    while (keys.hasMoreElements())
    {
      Object key = keys.nextElement();
      Object value = exitParam.get(key);

      if (!"[B".equals(value.getClass().getName()))
        System.out.println("  " + key + " = " + value);
      else
      {
        byte[] ba = (byte[]) value;
        System.out.println("  " + key + " = " + new String(ba) + " (" + ba.length + " bytes)");
      }
    }
  }

  private static boolean checkUserName(String userName)
  {
    // method to check if authentication credentials are denied or accepted
    boolean authenticated = false;
    if (userName.equals("Administrator") ||
        userName.equals("Ford Prefect")  ||
        userName.equals("Max Mustermann"))
    {
      authenticated = true;
    }
    if (userName.equals("Zaphod Beeblebrox") || userName.equals("Trillian"))
    {
      authenticated =  false;
    }
    return authenticated;
  }

  private static String mapUserName(String credentialUser)
  {
    // method to map the user name passed tothe exit to a MQWF user ID
    System.out.println("credentialUser in mapUserName() is set to: " + credentialUser);
    String mqwfID = "";
    if (credentialUser.equals("Administrator"))
    {
      // map to a valid MQWF user ID
      mqwfID = "Admin";
    }
    if (credentialUser.equals("Ford Prefect"))
    {
      // map to a unknown user ID -> results in an unknown user exception
      mqwfID = "ZYUWLM";
    }
    if (credentialUser.equals("Zaphod Beeblebrox") || credentialUser.equals("Trillian"))
    {
      // as these users are never authenticated, their user ID remains empty!
      mqwfID = "";
    }
    if (credentialUser.equals("Max Mustermann"))
    {
      // map to valid user ID
      mqwfID = "admin";
    }

    return mqwfID;
  }

  private static String extractCN(String certSubject)
  {
    // Extract the name of the user from the subject entry of the certificate
    // Because the subject entry of a X509 certificate has the following form:
    // ' CN=<username>, OU=....' the Strings 'CN=' and ',' encapsulate the user name.
    // Therefore the substring between these strings is returned.
    String temp = certSubject;
    // System.out.println("SubjectString: " + temp);
    temp = temp.substring(temp.indexOf("CN=")+3, temp.indexOf(",")); //+3 to cut off CN=
    // System.out.println("extracted CN value = " + temp);
    return temp;
  }
}
