/*BEGIN COPYRIGHT */
/**********************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**********************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/*********************************************************************/
/* MQSeries Workflow API Tutorial - Persistent lists and service     */
/*                                  queries                          */
/*                                                                   */
/* This program demonstrates                                         */
/* 1. the handling of persistent lists. Defining a persistent list   */
/*    means defining a persistent view to a collection of objects.   */
/* 2. the handling of process template lists, process instance lists */
/*    and worklists queried from the execution service               */
/*********************************************************************/

// Import necessary MQWF Java API Interface

import com.ibm.workflow.api.*;
import com.ibm.workflow.api.PersistentListPackage.*;
import com.ibm.workflow.api.ServicePackage.*;


// Various needed classes

import java.io.*;
import java.util.*;

public class Lists_Queries {

public static void main(String args[])

{
  int i = 0, j = 0, threshold =0;

  boolean del = false;

  BufferedReader in = new BufferedReader(new InputStreamReader(System.in));


/*--------------------------------------------------------------------*/
/* Check command line parameters, logon to system                     */
/*--------------------------------------------------------------------*/

  if (args.length<4)
  {
    System.out.println("\nUsage: java Lists_Queries <name of agent> <loc|jndi> <UserID> <Password> [-d]");
    System.exit(-1);
  }

  try { // general try clause for whole example

  Agent agent = new Agent();
  if      (args[1].equalsIgnoreCase("loc"))  agent.setLocator(Agent.LOC_LOCATOR);
  else if (args[1].equalsIgnoreCase("jndi")) agent.setLocator(Agent.JNDI_LOCATOR);
  else { System.out.println("Invalid locator policy: " + args[1]);
         System.exit(-1); }
  agent.setName(args[0]);
  ExecutionService service = agent.locate("","");
  System.out.println();
  service.logon2( args[2], args[3], SessionMode.DEFAULT, AbsenceIndicator.LEAVE );
  System.out.println("Logged on with UserID '" + service.userID() +
                     "' on system '" + service.systemName() +
                     "' at system group '" + service.systemGroupName() + "'.");

  if (args.length==5) if (args[4].equals("-d")) del=true;
  String user = service.userID();


/*********************************************************************/
/* Demonstrate handling of process instance lists.                   */
/*                                                                   */
/* Create process instance lists with different properties suited    */
/* for different scenarios. If a list with the name 'Sample*' already*/
/* exists, it will be deleted before list creation starts.           */
/*********************************************************************/

  ProcessInstanceList   instanceList;
  ProcessInstanceList[] iList;
  ProcessInstance       processInstance;
  ProcessInstance[]     viList;
  String listName ="";

/*-------------------------------------------------------------------*/
/* Clean up existing process instance lists.                         */
/*-------------------------------------------------------------------*/

  iList = service.queryProcessInstanceLists();

  for( i=0; i< iList.length; i++ )
  {
    instanceList = iList[i];
    listName = iList[i].name();

    if (listName.startsWith("Sample"))
    {
      try { instanceList.delete(); }
      catch (FmcException e)
      { System.out.println("Deletion of process instance list '"
                           + listName + "' failed with RC "
                           + e.rc + " - " + e.messageText); }

      if (del) System.out.println("\nProcess instance list deleted: " + listName);
    }
  }

/*-------------------------------------------------------------------*/
/* Create process instance lists for different purposes.             */
/* 1. Create a list which will contain only running process          */
/*    instances when queried and which is accessible by everyone.    */
/*    Show contents of created list.                                 */
/* 2. Create a list containing the latest started process instance.  */
/*    This list is for the personal use of the logged on user only.  */
/*    Show contents of created list.                                 */
/*-------------------------------------------------------------------*/

  if (!del) {                        // Only create lists when -d switch
                                                       // was not used
  threshold = 5;
  listName = "Sample_RunningProcessInstances";
  String desc = "Sample list of all process instances which are currently running";
  String PIListFilter = "STATE = RUNNING";
  String PIListSort     = "NAME ASC";
  try { instanceList = service.createProcessInstanceList( listName,
                                         TypeOfList.PUBLIC,
                                         "",
                                         desc,
                                         PIListFilter,
                                         PIListSort,
                                         new Integer(threshold) ); }
  catch (FmcException e)
  { System.out.println("Creation of public process instance list '"
                       + listName + "' failed with RC " + e.rc
                       + " - " + e.messageText); }
  System.out.println("\nPublic process instance list created: " + listName);

  // Query process instance lists again

  iList = service.queryProcessInstanceLists();

  // Select the newly created process instance list

  if (iList.length > 0 )
  {
    for( i=0; i<iList.length; i++ )
    {
      instanceList = iList[i];
      listName = instanceList.name();

      if (listName.equals("Sample_RunningProcessInstances"))
      {
       System.out.println("\nShowing process instances matched by this list:\n");

       viList = instanceList.queryProcessInstances();

       if ( viList.length > 0 )
       {
         for ( j=0; j<viList.length; j++ )
         {
           System.out.println("     " + viList[j].name());
         }
       }
       else System.out.println("     <none>");

      }
    }
   }

/*-------------------------------------------------------------------*/

   listName = "Sample_LatestStartedInstance";
   desc = "Sample list which contains only the latest started process instance";
   PIListSort = "START_TIME DESC";
   threshold= 1;

   try { instanceList = service.createProcessInstanceList( listName,
                                          TypeOfList.PRIVATE,
                                          user,
                                          desc,
                                          "",
                                          PIListSort,
                                          new Integer(threshold) ); }
  catch (FmcException e)
  { System.out.println("Creation of private process instance list '"
                       + listName + "' for user '" + user + "' failed with RC " + e.rc
                       + " - " + e.messageText); }
  System.out.println("\nPrivate process instance list for '" + user
             + "' created: " + listName);

  } // end if: del==false

  // Query process instance lists again

  iList = service.queryProcessInstanceLists();

  // Select the newly created process instance list

  for( i=0; i< iList.length; i++ )
  {
    instanceList = iList[i];
    listName = instanceList.name();

    if (listName.equals("Sample_LatestStartedInstance"))
    {
      System.out.println("\nShowing process instance matched by this list:\n");

      viList = instanceList.queryProcessInstances();

      if ( viList.length > 0 )
      {
        for ( j=0; j<viList.length; j++ )
        {
          System.out.println("     " + viList[j].name());
        }
      }
      else System.out.println("     <none>");
    }
  }


/*********************************************************************/
/* Demonstrate handling of worklists.                                */
/*                                                                   */
/* Create worklists with different properties suited                 */
/* for different scenarios. If a list with the name 'Sample*' already*/
/* exists, it will be deleted before list creation starts.           */
/*********************************************************************/

  WorkList   itemList;
  WorkList[] wList;
  WorkItem   workitem;
  WorkItem[] vwList;

/*-------------------------------------------------------------------*/
/* Clean up existing worklists                                       */
/*-------------------------------------------------------------------*/

  wList = service.queryWorkLists();

  for( i=0; i< wList.length; i++ )
  {
    itemList = wList[i];
    listName = itemList.name();

    if (listName.startsWith("Sample"))
    {
      try { itemList.delete(); }
      catch (FmcException e)
      { System.out.println("Deletion of worklist '"
                           + listName + "' failed with RC "
                           + e.rc + " - " + e.messageText); }

      if (del) System.out.println("\nWorklist deleted: " + listName);
    }
  }

/*-------------------------------------------------------------------*/
/* Create worklists for different purposes.                          */
/* 1. Create a list containing notification items only.              */
/*    This list is intended for public use and has no owner.         */
/* 2. Create a list containing the user's oldest 20 workitems.       */
/*    This list is for the personal use of the logged on user only.  */
/*-------------------------------------------------------------------*/

  if (!del) {                        // only create lists when -d switch
                                     // was not used
  listName = "Sample_NotificationList";
  String desc = "Sample list of all notifications for the logged on user.";
  String WIListFilter = "TYPE IN (PROCESS_NOTIFICATION,FIRST_NOTIFICATION,SECOND_NOTIFICATION)";
  String WIListSort = "PRIORITY ASC";
  try { itemList = service.createWorkList( listName,
                               TypeOfList.PUBLIC,
                               "",
                               desc,
                               WIListFilter,
                               WIListSort,
                               null ); }
  catch (FmcException e)
  { System.out.println("Creation of public worklist '"
                       + listName + "' failed with RC " + e.rc
                       + " - " + e.messageText); }
  System.out.println("\nPublic worklist created: " + listName);

  // Query worklists again

  wList = service.queryWorkLists();

  // Select the newly created worklist

  for( i=0; i<wList.length; i++ )
  {
    itemList = wList[i];
    listName = itemList.name();

    if (listName.equals("Sample_NotificationList"))
    {
      System.out.println("\nShowing workitem matched by this list:\n");

      vwList = itemList.queryWorkItems();

      if ( vwList.length > 0 )
      {
        for ( j=0; j<vwList.length; j++ )
        {
          System.out.println("     " + vwList[j].name());
        }
      }
      else System.out.println("     <none>");
    }
  }

/*-------------------------------------------------------------------*/

  listName = "Sample_MyWork";
  desc = "Sample list of the logged on user's workitems";
  WIListSort = "LAST_MODIFICATION_TIME DESC";
  threshold= 20;

  try { itemList = service.createWorkList( listName,
                              TypeOfList.PRIVATE,
                              user,
                              desc,
                              "",
                              WIListSort,
                              new Integer(threshold) ); }
  catch (FmcException e)
  { System.out.println("Creation of private worklist '"
                       + listName + "' for user '" + user + "' failed with RC " + e.rc
                       + " - " + e.messageText); }
  System.out.println("\nPrivate worklist for '" + user
             + "' created: " + listName);

  } // end if: del==false

  // Query worklists again

  wList = service.queryWorkLists();

  // Select the newly created worklist

  for( i=0; i< wList.length; i++ )
  {
    itemList = wList[i];
    listName = itemList.name();

    if (listName.equals("Sample_MyWork"))
    {
      System.out.println("\nShowing workitem matched by this list:\n");

      vwList = itemList.queryWorkItems();

      if ( vwList.length > 0 )
      {
        for ( j=0; j<vwList.length; j++ )
        {
          System.out.println("     " + vwList[j].name());
        }
      }
      else System.out.println("     <none>");

    }
  }


/*********************************************************************/
/* Demonstrate querying process templates                            */
/*                                                                   */
/* Query process templates with different properties suited          */
/* for different scenarios.                                          */
/*********************************************************************/

  ProcessTemplate   processTemplate;
  ProcessTemplate[] tList;

/*-------------------------------------------------------------------*/
/* 1. Display all existing process templates                         */
/* 2. Display a selection of process templates using filter and      */
/*    threshold criteria.                                            */
/*-------------------------------------------------------------------*/

  System.out.println("\nDisplaying all existing process templates sorted by 'NAME ASC':\n");

  String PTListSort = "NAME ASC";

  tList = service.queryProcessTemplates( "", PTListSort, null );

  if (tList.length > 0 )
  {
    for( i=0; i<tList.length; i++ )
    {
      processTemplate = tList[i];
      System.out.println("     " + processTemplate.name());
    }
  }
  else System.out.println("     <none>");

  System.out.println("\nDisplaying process templates with filter "
       + "'NAME='CreditRequest',\n" + "threshold " + threshold
       + " and sort criterium 'NAME DESC'.\n");

  String PTListFilter = "NAME='CreditRequest'";
  PTListSort   = "NAME DESC";

  tList = service.queryProcessTemplates( PTListFilter,
                                      PTListSort,
                                      new Integer(threshold) );

  if ( tList.length>0 )
  {
    for( i=0; i<tList.length; i++ )
    {
      processTemplate = tList[i];
      System.out.println("     " + processTemplate.name());
    }
  }
  else System.out.println("     <none>");


/*********************************************************************/
/* Demonstrate querying process instances                            */
/*                                                                   */
/* Query process instances with different properties suited          */
/* for different scenarios.                                          */
/*********************************************************************/

/*-------------------------------------------------------------------*/
/* 1. Display all existing process instances                         */
/* 2. Display a selection of process instances using filter and      */
/*    threshold criteria.                                            */
/*-------------------------------------------------------------------*/


  System.out.println("\nDisplaying all process instances sorted by 'NAME ASC':\n");

  String PIListSort   = "NAME ASC";

  viList = service.queryProcessInstances( "", PIListSort, null );

  if (viList.length > 0 )
  {
    for( i=0; i<viList.length; i++ )
    {
      processInstance = viList[i];
      System.out.println("     " + processInstance.name());
    }
  }
  else System.out.println("     <none>");

  System.out.println("\nDisplaying process instances with filter "
       + "'STATE IN (RUNNING)',\n" + "threshold " + threshold
       + " and sort criterium 'START_TIME DESC'.\n");

  String PIListFilter = "STATE IN (RUNNING)";
  PIListSort   = "START_TIME DESC";

  viList = service.queryProcessInstances( PIListFilter,
                                      PIListSort,
                                      new Integer(threshold));

  if ( viList.length>0 )
  {
    for( i=0; i<viList.length; i++ )
    {
      processInstance = viList[i];
      processInstance.refresh();

      System.out.println(processInstance.creationTime().getTime().toString() + "   "
           + processInstance.name());
    }
  }
  else System.out.println("     <none>");


/*********************************************************************/
/* Demonstrate querying workitems                                    */
/*                                                                   */
/* Query workitems with different properties suited                  */
/* for different scenarios.                                          */
/*********************************************************************/

/*-------------------------------------------------------------------*/
/* 1. Display all existing workitems                                 */
/* 2. Display a selection of process instances using filter and      */
/*    threshold criteria.                                            */
/*-------------------------------------------------------------------*/

  System.out.println("\nDisplaying all workitems sorted by 'NAME ASC':\n");

  String WIListSort   = "NAME ASC";

  vwList = service.queryWorkItems( "", WIListSort, null );

  if ( vwList.length>0 )
  {
    for( i=0; i<vwList.length; i++ )
    {
      workitem = vwList[i];
      System.out.println("     " + workitem.name());
    }
  }
  else System.out.println("     <none>");

  threshold = 10;

  System.out.println("\nDisplaying workitems with filter "
       + "'STATE IN (READY)',\n" + "threshold " + threshold
       + " and sort criterium 'PROCESS_NAME ASC'.\n");

  String WIListFilter = "STATE IN (READY)";
  WIListSort   = "PROCESS_NAME ASC";

  vwList = service.queryWorkItems( WIListFilter,
                               WIListSort,
                               new Integer (threshold) );

  if ( vwList.length>0 )
  {
    for( i=0; i<vwList.length; i++ )
    {
      workitem = vwList[i];
      System.out.println("     " + workitem.name());
    }
  }
  else System.out.println("     <none>");


/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/

  System.out.println("\nLogging off.");
  service.logoff();
  System.exit(0);

  } // end of general try clause

  catch(FmcException e)
  {
    // Catch and report details about the FmcException
    System.out.println("FmcException occured");
    System.out.println("  RC         : " + e.rc);
    System.out.println("  Origin     : " + e.origin);
    System.out.println("  MessageText: " + e.messageText);
    System.out.println("  Exception  : " + e.getMessage());
    System.out.println("  Parameters : ");
    for ( i = 0; i < e.parameters.length ; i++)
    {
      System.out.println("    " + e.parameters[i] );
    }
    System.out.println("  StackTrace : ");
    e.printStackTrace();
  }
  catch(Exception e)
  {
    // Catch and report any exception that occurred.
    e.printStackTrace();
  }

} // end of void main

} // end of Lists_Queries
